/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file settings.h
\brief Settings object for plugins \see Settings

\author Bartosz Szatkowski <bulislaw@linux.com>
*/

#ifndef SETTINGS_H
#define SETTINGS_H

#include <QString>
#include <QHash>
#include <QDebug>

/*! Plugins or dictionaries may need to keep some of configuration between
  sessions, moreover Backbone or GUI may want to store some additional info in
  plugin Settings.

  It's important for plugin to store all information given to it in Settings.*/
class Settings {
  public:
    Settings() {}
    ~Settings() {}
    Settings(const Settings* set) {
        _settings = QHash<QString, QString>(set->_settings);
    }

    /*!
        \returns value fo a given key
        \param key name of a given setting
        \return value of a given setting
    */
    QString value(const QString key) const {
        if(!_settings.contains(key)) {
            return QString();
        }
        return _settings[key];
    }

    /*! sets key to a value
        \param key name of a given setting
        \param value value of a given setting
    */
    void setValue(const QString key, const QString value) {
        _settings.insert(key, value);
    }

    //! \returns list of keys (QList<QString>)
    QList<QString> keys() const {
        return _settings.keys();
    }

private:
    //! keeps the settings
    QHash<QString, QString> _settings;
};

#endif // SETTINGS_H
