/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file TranslationWidget.cpp
    \brief Displays translation of a word found in dictionaries

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "TranslationWidget.h"
#include <QDebug>
#ifdef Q_WS_MAEMO_5
    #include <QtGui/QX11Info>
    #include <X11/Xlib.h>
    #include <X11/Xatom.h>
#endif
#include <QWebFrame>

TranslationWidget::TranslationWidget(QWidget *parent):
        QScrollArea(parent){

    guiinterface = qobject_cast<GUIInterface*>(parent);

    #ifdef Q_WS_MAEMO_5
        setAttribute(Qt::WA_Maemo5StackedWindow);
        setWindowFlags(windowFlags() | Qt::Window);
    #endif

    initializeUI();

    setWindowTitle("mDictionary");

    connect(webkit, SIGNAL(search()),
           this, SLOT(searchSelected()));

    QFile file(":/xsl/xsl.xsl");
    if(!file.open(QFile::ReadOnly))
        qDebug()<<"error can't open a xslt file";
    else
        xslt=file.readAll();
}


void TranslationWidget::show() {
    QScrollArea::show();
}


void TranslationWidget::show(QStringList translations) {
    showMaximized();

    #ifdef Q_WS_MAEMO_5
        if(!buttonsInitialized)
            initButtons();
    #endif

    QString trans= createTranslation(translations);

    QString head = "<html><head>";
    head += "<link rel=\"stylesheet\" type=\"text/css\" "
            "href=\"/usr/share/mdictionary/css/" "style.css"
            "\" /></head><body ondragstart=\"return false\">";
    trans = head + trans + "</body></html>";

    trans.replace("$STAR$", "/usr/share/mdictionary/staron.png");

    webkit->setHtml(trans, QUrl().fromLocalFile("/"));
}


QString TranslationWidget::createTranslation(QStringList translations) {
    QString trans;
    if(!translations.size())
        return QString("Empty translation");
    int element=-1;
    do{
        int a=0;
        foreach(QString t, translations){
            if(a!=element)
                trans += t + "\n";
            a++;
        }
        trans=QString("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") + QString("\n <ar>")
               + trans + QString("\n </ar>");

        trans=checkTranslation(trans);
        trans=XslConversion(trans);
        element++;
        if(element>translations.size())
            return QString("Empty translation");
    } while(trans==""); // trans=="" only when XslConversation have a error;
    return trans;
}


QString TranslationWidget::XslConversion(QString translation)
{
    QXmlQuery myQuery(QXmlQuery::XSLT20);
    myQuery.setFocus(translation);
    myQuery.setQuery(xslt);
    QString result("");
    myQuery.evaluateTo(&result);
    return result;
}


QString TranslationWidget::checkTranslation(QString trans){
    /*check all "&"*/
    int pos=0;
    int pos2;
    int count=0;

    /*check "<" and ">" in key */
    while(pos!=-1){
        pos=trans.indexOf("&",pos+1);
        pos2=trans.indexOf("amp;",pos);
        if(pos!=-1 && pos2!=-1 && pos+1==pos2)
            continue;
        pos2=trans.indexOf("gt;",pos);
        if(pos!=-1 && pos2!=-1 && pos+1==pos2)
            continue;
        pos2=trans.indexOf("lt;",pos);
        if(pos!=-1 && pos2!=-1 && pos+1==pos2)
            continue;
        pos2=trans.indexOf("apos;",pos);
        if(pos!=-1 && pos2!=-1 && pos+1==pos2)
            continue;
        pos2=trans.indexOf("quot;",pos);
        if(pos!=-1 && pos2!=-1 && pos+1==pos2)
            continue;
        if(pos!=-1)
            trans.insert(pos+1,"amp;");
    }

    /*check "<" and ">" in key */
    pos=0;
    count=0;
    while(pos<trans.size()){
        if(pos==trans.indexOf("<key>",pos)){
            count++;
            pos+=4;
        }
        else if(pos==trans.indexOf("</key>",pos))
            count--;
        else if(count==1 && (trans.at(pos)=='<' ||  trans.at(pos)=='>' ))
            trans.remove(pos,1);
        pos++;
    }

    /*checking unclosed tag's*/
    pos=0;
    count=0;
    while(pos<trans.size()){
        if(trans.at(pos)=='<')
            count++;
        else if(trans.at(pos)=='>')
            count--;
        if(count==-1){     // remove ">"
            trans.remove(pos,1);
            count=0;
        }
        pos++;
    }
    pos=trans.size()-1;
    count=0;
    while(pos>-1){
        if(trans.at(pos)=='>')
            count++;
        else if(trans.at(pos)=='<')
            count--;
        if(count==-1){     // remove "<"
            trans.remove(pos,1);
            count=0;
        }
        pos--;
    }

    /*check """ and "'" */
    pos=0;
    count=0;
    while(pos<trans.size()){
        if(trans.at(pos)=='<')
            count++;
        else if(trans.at(pos)=='>')
            count--;
        else if(trans.at(pos)=='\"' && count==0)
            trans.replace(pos,1,"&quot;");
        else if(trans.at(pos)=='\'' && count==0)
            trans.replace(pos,1,"&apos;");
        pos++;
    }
    return trans;
}


#ifdef Q_WS_MAEMO_5
void TranslationWidget::initButtons() {
        int x = width() - showButtonsButton->sizeHint().width();
        int y = height() - showButtonsButton->sizeHint().height();

        showButtonsButton->move(QPoint(x,y));
        showButtonsButton->show();

        x = width() - zoomOutButton->sizeHint().width();
        y = height() - 2*zoomOutButton->sizeHint().height();
        zoomOutButton->move(QPoint(x, height()));

        zoomOutButtonAnimation = new QPropertyAnimation(zoomOutButton,
                                                        "pos", this);
        zoomOutButtonAnimation->setStartValue(QPoint(x, height()));
        zoomOutButtonAnimation->setEndValue(QPoint(x,y));
        zoomOutButtonAnimation->setDuration(200);
        zoomOutButtonAnimation->setEasingCurve(QEasingCurve::InOutBack);

        x = width() - zoomInButton->sizeHint().width();
        y = height() - 3*zoomInButton->sizeHint().height();
        zoomInButton->move(QPoint(x, height()));

        zoomInButtonAnimation = new QPropertyAnimation(zoomInButton,
                                                      "pos", this);
        zoomInButtonAnimation->setStartValue(QPoint(x, height()));
        zoomInButtonAnimation->setEndValue(QPoint(x,y));
        zoomInButtonAnimation->setDuration(400);
        zoomInButtonAnimation->setEasingCurve(QEasingCurve::InOutBack);

        x = 0;
        y = height() - copyButton->sizeHint().height();

        copyButton->move(QPoint(x, height()));
        copyButtonAnimation = new QPropertyAnimation(copyButton, "pos", this);
        copyButtonAnimation->setStartValue(QPoint(x, height()));
        copyButtonAnimation->setEndValue(QPoint(x,y));
        copyButtonAnimation->setDuration(200);
        copyButtonAnimation->setEasingCurve(QEasingCurve::InOutBack);

        x = 0;
        y = height() - 2*copyButton->sizeHint().height();

        selectAllButton->move(QPoint(x, height()));
        selectAllButtonAnimation = new QPropertyAnimation(selectAllButton,
                                                         "pos", this);
        selectAllButtonAnimation->setStartValue(QPoint(x, height()));
        selectAllButtonAnimation->setEndValue(QPoint(x,y));
        selectAllButtonAnimation->setDuration(400);
        selectAllButtonAnimation->setEasingCurve(QEasingCurve::InOutBack);

        x = 0;
        y = height() - 3*copyButton->sizeHint().height();

        searchButton->move(QPoint(x, height()));
        searchButtonAnimation = new QPropertyAnimation(searchButton,
                                                      "pos", this);
        searchButtonAnimation->setStartValue(QPoint(x, height()));
        searchButtonAnimation->setEndValue(QPoint(x,y));
        searchButtonAnimation->setDuration(600);
        searchButtonAnimation->setEasingCurve(QEasingCurve::InOutBack);

        buttonsAnimation = new QParallelAnimationGroup(this);
        buttonsAnimation->addAnimation(zoomInButtonAnimation);
        buttonsAnimation->addAnimation(zoomOutButtonAnimation);
        buttonsAnimation->addAnimation(selectAllButtonAnimation);
        buttonsAnimation->addAnimation(copyButtonAnimation);
        buttonsAnimation->addAnimation(searchButtonAnimation);
        buttonsInitialized = true;
        buttonsVisible = false;

        connect(showButtonsButton, SIGNAL(clicked()),
                this, SLOT(showButtons()));
}
#endif

void TranslationWidget::initializeUI() {
    webkit = new TranslationView(this);
    Settings* set = guiinterface->settings();
    qreal fac = set->value("zoom").toFloat();
    if(!fac)
        fac++;
    webkit->setZoomFactor(fac);
    delete set;

    QWidget*w = new QWidget(this);
    verticalLayout = new QVBoxLayout(w);
    verticalLayout->addWidget(webkit);

    this->setWidget(w);
    this->setWidgetResizable(true);

    #ifdef Q_WS_MAEMO_5
        zoomInButton = new QToolButton(this);
        zoomInButton->setIcon(QIcon::fromTheme("pdf_zoomin"));
        zoomInButton->setMinimumSize(zoomInButton->sizeHint());

        zoomOutButton = new QToolButton(this);
        zoomOutButton->setIcon(QIcon::fromTheme("pdf_zoomout"));
        zoomOutButton->setMinimumSize(zoomOutButton->sizeHint());

        selectAllButton = new QToolButton(this);
        selectAllButton->setIcon(QIcon(":/icons/48x48/edit-select-all.png"));
        selectAllButton->setMinimumSize(selectAllButton->sizeHint());

        copyButton = new QToolButton(this);
        copyButton->setIcon(QIcon::fromTheme("general_notes"));
        copyButton->setMinimumSize(copyButton->sizeHint());

        searchButton = new QToolButton(this);
        searchButton->setIcon(QIcon::fromTheme("general_search"));
        searchButton->setMinimumSize(searchButton->sizeHint());

        showButtonsButton = new QToolButton(this);
        showButtonsButton->setIcon(QIcon::fromTheme("general_sent"));
        showButtonsButton->setMinimumSize(searchButton->sizeHint());

        connect(zoomInButton, SIGNAL(clicked()),
                webkit, SLOT(zoomIn()));

        connect(zoomOutButton, SIGNAL(clicked()),
                webkit, SLOT(zoomOut()));


        connect(searchButton, SIGNAL(clicked()),
                this, SLOT(searchSelected()));

        connect(copyButton, SIGNAL(clicked()),
                this, SLOT(copySelected()));

        connect(selectAllButton, SIGNAL(clicked()),
                webkit, SLOT(selectAll()));

        buttonsInitialized = false;



        grabZoomKeys(true);
    #endif
}

void TranslationWidget::searchSelected() {
    if(webkit->selectedText().isEmpty()) {
        Q_EMIT notify(Notify::Warning, tr("No text is selected"));
    }
    else {
        #ifdef Q_WS_MAEMO_5
            hide();
        #endif
        Q_EMIT search(webkit->selectedText().toLower());
    }
}

void TranslationWidget::copySelected() {
    if(webkit->selectedText().isEmpty()) {
        Q_EMIT notify(Notify::Warning, tr("No text is selected"));
    }
    else {
        Q_EMIT notify(Notify::Info, tr("Copyied selected text"));
    }
}

#ifdef Q_WS_MAEMO_5
void TranslationWidget::showButtons() {
    if(!buttonsVisible) {
        buttonsAnimation->setDirection(QAbstractAnimation::Forward);
        buttonsAnimation->start();
        buttonsVisible = true;

        showButtonsButton->setIcon(QIcon::fromTheme("general_received"));
    }
    else if(buttonsVisible) {
        buttonsAnimation->setDirection(QAbstractAnimation::Backward);
        buttonsAnimation->start();
        buttonsVisible = false;
        showButtonsButton->setIcon(QIcon::fromTheme("general_sent"));
    }
}

void TranslationWidget::grabZoomKeys(bool grab) {
     if (!winId()) {
         return;
     }

    unsigned long val = (grab) ? 1 : 0;
    Atom atom = XInternAtom(QX11Info::display(),
                            "_HILDON_ZOOM_KEY_ATOM", False);
    if (!atom) {
        return;
    }

    XChangeProperty (QX11Info::display(),
         winId(),
         atom,
         XA_INTEGER,
         32,
         PropModeReplace,
         reinterpret_cast<unsigned char *>(&val),
         1);
}

void TranslationWidget::hideEvent(QHideEvent* e) {
    if(buttonsVisible)
        showButtons();

    QScrollArea::hideEvent(e);
}

void TranslationWidget::keyPressEvent(QKeyEvent* event) {
    switch (event->key()) {
        case Qt::Key_F7:
        webkit->zoomIn();
        event->accept();
        break;

        case Qt::Key_F8:
        webkit->zoomOut();
        event->accept();
        break;
    }
    QWidget::keyPressEvent(event);
}
#endif


void TranslationWidget::updateZoom(qreal factor) {
    Settings* set = guiinterface->settings();
    set->setValue("zoom", QString("%1").arg(factor));
    guiinterface->setSettings(set);
}



