/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file XdxfSettingsDialog.cpp
*/
//Created by Mateusz Półrola

#include "XdxfSettingsDialog.h"
#include <QDebug>

XdxfSettingsDialog::XdxfSettingsDialog(XdxfPlugin *plugin, QWidget *parent) :
    QDialog(parent)
{
    this->plugin = plugin;
    verticalLayout = new QVBoxLayout();
       setLayout(verticalLayout);

    setWindowTitle(tr("XDXF Settings"));


    infoLabel = new QLabel(this);

    infoLabel->setText(tr("Plugin: ") + plugin->type().toUpper() +"\n" +
                   tr("From: ") + plugin->langFrom() + "\n" +
                   tr("To: ") + plugin->langTo() + "\n" +
                   tr("Description: ") + plugin->name());

    verticalLayout->addWidget(infoLabel);

    browseLayout = new QHBoxLayout();
    verticalLayout->addLayout(browseLayout);

    browseButton =  new QPushButton(tr("Browse"));
    browseLabel = new QLabel(tr("Dictionary file: ") +
                             plugin->settings()->value("path"));

    browseLayout->addWidget(browseLabel);
    browseLayout->addWidget(browseButton,0, Qt::AlignRight);


    cacheLayout = new QHBoxLayout();
    verticalLayout->insertLayout(-1,cacheLayout,0);
    accentsCheckBox = new QCheckBox(tr("Strip accents \n(searching takes more time, "
                 "but spelling don't have to be exact)"));
    verticalLayout->addWidget(accentsCheckBox);

    if(plugin->settings()->value("strip_accents") == "true")
        accentsCheckBox->setChecked(true);
    else
        accentsCheckBox->setChecked(false);

    cacheCheckBox = new QCheckBox(tr("Optimize for quicker searches (may take some time)"),this);
    if(plugin->settings()->value("cached") == "true") {
        cacheCheckBox->setChecked(true);
        accentsCheckBox->setChecked(true);
        accentsCheckBox->setEnabled(false);
        _generateCache = true;
    }
    else {
        cacheCheckBox->setChecked(false);
        _generateCache = false;
    }

    cacheLayout->addWidget(cacheCheckBox);

    saveButton = new QPushButton(tr("Save settings"));

    verticalLayout->addWidget(saveButton);

    setModal(true);

    connect(browseButton, SIGNAL(clicked()),
            this, SLOT(selectFile()));

    connect(saveButton, SIGNAL(clicked()),
            this, SLOT(accept()));

    connect(cacheCheckBox, SIGNAL(toggled(bool)),
            SLOT(setGenerateCache(bool)));

    connect(accentsCheckBox, SIGNAL(clicked(bool)), SLOT(setAccents(bool)));


    _dicitonaryFilePath = plugin->settings()->value("path");
    lastAccents = accentsCheckBox->isChecked();
}


void XdxfSettingsDialog::setAccents(bool state) {
    lastAccents = state;
}


void XdxfSettingsDialog::setGenerateCache(bool generate) {
    _generateCache = generate;

    if(generate)
        accentsCheckBox->setChecked(true);
    else
        accentsCheckBox->setChecked(lastAccents);

    accentsCheckBox->setEnabled(!generate);
}

bool XdxfSettingsDialog::generateCache() {
    return _generateCache;
}

void XdxfSettingsDialog::selectFile() {
    QString fileName = QFileDialog::getOpenFileName(this,
                                     tr("Select dictionary file"),
                                     "",
                                     tr("XDXF Files (*.xdxf)"),
                                     0,
                                     0);
    if (!fileName.isEmpty()) {
        browseLabel->setText(tr("Dictionary file: ") + fileName);
        _dicitonaryFilePath = fileName;
    }    
}

QString XdxfSettingsDialog::dicitonaryFilePath() {
    return _dicitonaryFilePath;
}

Settings* XdxfSettingsDialog::getSettings(XdxfPlugin *plugin,
                                          QWidget *parent) {
    XdxfSettingsDialog settingsDialog(plugin, parent);


    if(settingsDialog.exec()==QDialog::Accepted) {
        Settings* settings = new Settings;
        foreach(QString key, plugin->settings()->keys())
            settings->setValue(key, plugin->settings()->value(key));
        settings->setValue("path", settingsDialog.dicitonaryFilePath());

        if(settingsDialog.generateCache()) {
            settings->setValue("generateCache", "true");
        }
        else {
            settings->setValue("generateCache", "false");
        }

        if(settingsDialog.accentsCheckBox->isChecked())
            settings->setValue("strip_accents", "true");
        else
            settings->setValue("strip_accents", "false");

        plugin->setSettings(settings);
        delete settings;
        return 0;
    }

    return 0;
}


