/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file DictsProxyModel.h
    \brief roxy model used for filtering and sorting informations about xdxf dictionaries.

    \author Mateusz Półrola <mateusz.polrola@gmail.com>
*/

#ifndef DICTSPROXYMODEL_H
#define DICTSPROXYMODEL_H

#include <QSortFilterProxyModel>
#include <QDebug>


/*!
    Proxy model used for filtering and sorting informations about xdxf dictionaries.
    It allow to filter dictionaries based on their langages
*/
class DictsProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    DictsProxyModel(QObject *parent = 0): QSortFilterProxyModel(parent){

    }

    /*!
        Returns currently setted language used to filter dictionaries
        based on dictionarie's language from
    */
    QString from() { return _from;}

    /*!
        Returns currently setted language used to filter dictionaries
        based on dictionarie's language to
    */
    QString to() {return _to;}


    /*!
        Sets currently setted language used to filter dictionaries based
        on dictionarie's language from
    */
    void setFrom(QString from) { _from = from; invalidateFilter(); }

    /*!
        Sets currently setted language used to filter dictionaries
        based on dictionarie's language to
    */
    void setTo(QString to) {_to = to; invalidateFilter();}

protected:
    //! Filtering passed row
    bool filterAcceptsRow(int source_row, const QModelIndex&) const {
        QString sourceFrom = sourceModel()->data(
                sourceModel()->index(source_row, 0)).toString();
        QString sourceTo = sourceModel()->data(
                sourceModel()->index(source_row, 1)).toString();

        return ((_from.isEmpty() || sourceFrom == _from) &&
                (_to.isEmpty() || sourceTo == _to));
    }

    /*!
        Sorting, if sort column is set to column containing size of dictionary,
        string containing size of it is converted to fload and compared to
        other dict size
    */
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const {
        if(sortColumn() == 3) {
            QString l = left.model()->data(left).toString();
            l.remove(" MB");

            QString r = right.model()->data(right).toString();
            r.remove(" MB");

            float lNumber = l.toFloat();
            float rNumber = r.toFloat();

            return (lNumber < rNumber);
        }
        else
            QSortFilterProxyModel::lessThan(left, right);
    }

private:
    QString _from;
    QString _to;

};

#endif // DICTSPROXYMODEL_H
