/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
    \file TranslationXdxf.h
    \brief Implementation of xdxf plugin's translation.

    \author Jakub Jaszczynski <j.j.jaszczynski@gmail.com>
*/
#ifndef TRANSLATIONXDXF_H
#define TRANSLATIONXDXF_H

#include <QString>
#include "../../include/translation.h"
#include "xdxfplugin.h"

class TranslationXdxf : public Translation
{
public:
    TranslationXdxf();
    TranslationXdxf(QString _key,
                    QString _dictionaryInfo,
                    XdxfPlugin *xdxfPlugin);

    //! \return word to be translated
    QString key() const;

    //! \return parsed raw format into xml
    QString toXml() const;

    /*! sets the word for which we want to find a translation
        \param word for which we want to find a translation */
    void setKey(QString);

    //! sets information about dictionary
    void setDictionaryInfo(QString);

    //! \return whether given translation is taken from bookmarks
    int isBookmark() const {
        return _bookmark;
   }

    //! \return corresponding dict object
    uint dict() const {return _dictHash;}

private:
    QString _key;
    QString _dictionaryInfo;
    XdxfPlugin *xdxfPlugin;
    int _dictHash;
};

#endif // TRANSLATIONXDXF_H

